package com.lqkj.link.module.zone.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.lqkj.link.module.authority.repository.UserInfoRepository;
import com.lqkj.link.module.zone.domain.MaterialInfo;
import com.lqkj.link.module.zone.repository.MaterialInfoRepository;
import com.lqkj.link.module.zone.service.MaterialInfoService;
import com.lqkj.link.util.Unzipper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class MaterialInfoServiceImpl implements MaterialInfoService {

    @Autowired
    private MaterialInfoRepository materialInfoRepository;

    @Autowired
    private UserInfoRepository userInfoRepository;

    @Transactional
    @Override
    public void sava(MaterialInfo materialInfo) {
        materialInfo.setMaterialName(StringUtils.split(materialInfo.getCompressFileName(), ".")[0]);
        if ((Objects.nonNull(materialInfo.getMaterialId()) && materialInfoRepository.hasSameNameWithoutOne(materialInfo.getMaterialId(), materialInfo.getMaterialName()))
            || (Objects.isNull(materialInfo.getMaterialId()) && materialInfoRepository.hasSameName(materialInfo.getMaterialName()))) {
            throw new RuntimeException("材质名称不能重复!");
        }
        materialInfo.setUpdateTime(new Date());
        materialInfoRepository.save(materialInfo);
        userInfoRepository.updateRefreshStatus();
    }

    @Transactional
    @Override
    public void batchDelete(List<Integer> materialId) {
        materialInfoRepository.deleteAllById(materialId);
        // 更新用户资源刷新状态
        userInfoRepository.updateRefreshStatus();
    }


    @Override
    public Page<MaterialInfo> pageQuery(String name, Integer page, Integer pageSize, Integer materialType, Integer userId) {
        Pageable pageable = PageRequest.of(page, pageSize);
        return materialInfoRepository.pageQuery(name, materialType, pageable, userId);
    }

    @Transactional
    @Override
    public void batchAdd(String compressFilePath) {

        try {
            String unzipPath = "./upload/resource/material" + RandomUtil.randomNumbers(5) + "/";
            Unzipper.unZipFiles(new File("." + compressFilePath), unzipPath);
            Map<String, MaterialInfo> materialMap = materialInfoRepository.findAll().stream().collect(Collectors.toMap(MaterialInfo::getMaterialName, materialInfo -> materialInfo));

            String modelFolderPath = unzipPath + "model";
            File materialModelDir = new File(modelFolderPath);
            if (materialModelDir.isDirectory()){
                List<MaterialInfo> materialInfoList = new ArrayList<>();
                File[] jsonFiles = materialModelDir.listFiles();
                assert jsonFiles != null;
                for (File jsonFile : jsonFiles) {
                    if (jsonFile.isFile() && jsonFile.getName().endsWith(".json")) {
                        String modelFileName = jsonFile.getName();
                        String modelPath = unzipPath + "model/";
                        if (modelFileName.matches(".*[一-龥]+.*")) {
                            // 如果包含中文，重命名
                            String newFileName = UUID.randomUUID() + ".json";
                            modelPath += newFileName;
                            boolean rename = jsonFile.renameTo(new File(modelPath));
                            System.out.println(rename);
                            // 材质图标也要重命名
                            File oldIconFile = new File(unzipPath + "icon/" + modelFileName.replace(".json", ".png"));
                            if (oldIconFile.exists()) {
                                File newIconFile = new File(unzipPath + "icon/" + newFileName.replace(".json", ".png"));
                                rename = oldIconFile.renameTo(newIconFile);
                                System.out.println(rename);
                            }
                        } else {
                            modelPath += modelFileName;
                        }
                        if (!materialMap.containsKey(modelFileName.substring(0, modelFileName.lastIndexOf(".")))) {
                            materialInfoList.add(new MaterialInfo(null, modelFileName.substring(0, modelFileName.lastIndexOf(".")), modelPath.substring(1),
                                    modelPath.replace("model", "icon").replace("json", "png").substring(1), modelFileName, 1, new Date(), null));
                        } else {
                            MaterialInfo materialInfo = materialMap.get(modelFileName.substring(0, modelFileName.lastIndexOf(".")));
                            materialInfo.setMaterialIcon(modelPath.replace("model", "icon").replace("json", "png").substring(1));
                            materialInfo.setCompressFileName(modelFileName.substring(1));
                            materialInfo.setJsonPath(modelPath);
                            materialInfo.setUpdateTime(new Date());
                            materialInfoList.add(materialInfo);
                        }
                    }
                }
                materialInfoRepository.saveAll(materialInfoList);
            }
            userInfoRepository.updateRefreshStatus();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }


}
